/**
 * Project Salmon (Habbo Hotel r63b Emulator)
 *
 * Copyright (c) 2013 - 2014, Jacob Pollack.
 **
 * SVN: svn://jamach.servehttp.com/projectSalmon
 **
 */

#ifndef __SALMON_NET_CONNECTIONREACTOR_H
#define __SALMON_NET_CONNECTIONREACTOR_H

#include <map>

// Forward declaration of classes.
class Session;

class ConnectionReactor {
  private:
    // Stores the server socket's file descriptor.
    const int serverFD;
    // Stores the active connections.
    std::map<int, Session *> connections;
  public:
    // Constructs a ConnectionReactor object.
    explicit ConnectionReactor(const int);
    // Destructs a ConnectionReactor. Frees all active connections.
    ~ConnectionReactor();
  private:
    // Disables the copy constructor.
    explicit ConnectionReactor(const ConnectionReactor &);
    // Disables the assignment operator.
    ConnectionReactor &operator=(const ConnectionReactor &);
  public:
    // Handles the incoming connections and sends the non-blocked connection 
    // to the dispatcher.
    // :TODO: Ignore FDs 0, 1, and 2.
    void handleConnections();
    // Stores a new connection.
    void addConnection(Session * const);
    // Removes an existing connection.
    void removeConnection(Session * const);
};

#endif