/**
 * Project Salmon (Habbo Hotel r63b Emulator)
 *
 * Copyright (c) 2013 - 2014, Jacob Pollack.
 **
 * SVN: svn://jamach.servehttp.com/projectSalmon
 **
 */

#include "data/configdata.h"

#include <cstring>
#include <fstream>
#include <limits>
#include <stdexcept>

#include "exceptions/critexception.h"
#include "utils/constants.h"

using std::map;
using std::pair;

using std::ifstream;

using std::numeric_limits;
using std::streamsize;

using std::out_of_range;

namespace {
  // Determines if a string a is less than a string b in strict weak ordering.
  bool myStrCmp(char * const a, char * const b) {
    return (0 > strcmp(a, b)); // -1 >=
  }
}

// See header file.
ConfigData::ConfigData(const char * const configFile) {
  // Creates a new data map.
  this->data = map<char *, char *, bool (*)(char * const, char * const)>(&myStrCmp);

  // Temporary ifstream object. Used for parsing the configuration file.
  ifstream tmpifs;

  tmpifs.open(configFile, ifstream::in);

  // Determines if the file is open. If it is, parse the configuration data, 
  // otherwise log the error and perform a critical shutdown.
  if (true == tmpifs.is_open()) {
    // Stores the LHS and RHS of each line.
    char *bufferLHS = NULL;
    char *bufferRHS = NULL;

    // Iterates while the stream is in a good state.
    while (true == tmpifs.good()) {
      // Determines if the line starts with a hashtag. If so, ignore the line.
      // :TODO: Check for bad formatting of lines.
      if (('#' == tmpifs.peek()) || ('\n' == tmpifs.peek())) {
        tmpifs.ignore(numeric_limits<streamsize>::max(), '\n');
      } else {
        bufferLHS = new char[256]; // :TODO: Allow arbitrarily long lines.
        bufferRHS = new char[256]; // :TODO: Allow arbitrarily long lines.

        tmpifs.getline(bufferLHS, sizeof(char) * 256, '=');
        tmpifs.getline(bufferRHS, sizeof(char) * 256, '\n');

        this->data.insert(pair<char *, char *>(bufferLHS, bufferRHS));
      }
    }
  } else {
    throw CritException("Could not locate or read the configuration file.", __LINE__, __FILE__);
  }
}

// See header file.
ConfigData::~ConfigData() {
  // Iterate through the configuration data and clean up.
  for (map<char *, char *>::iterator it = this->data.begin(); it != this->data.end(); ++it) {
    delete[] it->first;
    delete[] it->second;
  }
}

// See header file.
char *ConfigData::operator[](const char * const key) {
  char *result = NULL;

  try {
    // Likely a better way to do this (i.e. without a const_cast).
    result = this->data.at(const_cast<char *>(key));
  // :TODO: Handle this error.
  } catch (out_of_range &oor) { }

  return result;
}