/**
 * Project Salmon (Habbo Hotel r63b Emulator)
 *
 * Copyright (c) 2013 - 2014, Jacob Pollack.
 **
 * SVN: svn://jamach.servehttp.com/projectSalmon
 **
 */

#include "net/session.h"

#include <errno.h>
#include <sys/socket.h>
#include <unistd.h>

#include "exceptions/critexception.h"

// See header file.
Session::Session(const int fileDes) : fileDes(fileDes) { }

// See header file.
Session::~Session() {
  // Closes the session socket's file descriptor.
  const int result = close(this->fileDes);

  // Determines if there was an error closing the session socket's file 
  // descriptor.
  if (-1 == result) {
    throw CritException("Could not close a session's file descriptor.", __LINE__, __FILE__);
  }
}

// See header file.
int Session::getFileDes() const {
  return this->fileDes;
}

// See header file.
bool Session::isAlive() {
  char buffer;

  // Peeks at one byte.
  const int result = recv(this->fileDes, &buffer, sizeof(char), MSG_PEEK);

  // Determines if there was an error receiving the message or if the client 
  // gracefully disconnected.
  if (0 >= result) {
    // Determines if there is nothing to read (i.e. it would normally block 
    // to read). If so, return true, otherwise return false.
    return (result == EWOULDBLOCK);
  }

  return true;
}
