/**
 * Project Salmon (Habbo Hotel r63b Emulator)
 *
 * Copyright (c) 2013 - 2014, Jacob Pollack.
 **
 * SVN: svn://jamach.servehttp.com/projectSalmon
 **
 */

#include "salmon.h"

#include "data/configdata.h"
#include "net/socketmanager.h"
#include "net/connectionreactor.h"
#include "utils/constants.h"
#include "utils/conversions.h"
#include "utils/threadpool.h"

using conversions::strtos;

// Defines the global variable, salmon, that is an instance of Salmon.
Salmon &salmon = Salmon::getInstance();

// Defines the static members of Salmon.
Salmon *Salmon::instance = NULL;

// See header file.
Salmon::Salmon() {
  this->configData = NULL;
  this->threadPool = NULL;
  this->socketManager = NULL;
}

// See header file.
Salmon::~Salmon() {
  delete this->socketManager;
  delete this->threadPool;
  delete this->configData;
}

// See header file.
Salmon &Salmon::getInstance() {
  // Determines if there is not already instance of Salmon. If not, create a 
  // Salmon object.
  if (NULL == Salmon::instance) {
    Salmon::instance = new Salmon();
  }

  return *(Salmon::instance);
}

// See header file.
ConfigData &Salmon::getConfigData() const {
  return *(this->configData);
}

// See header file.
ThreadPool &Salmon::getThreadPool() const {
  return *(this->threadPool);
}

// See header file.
SocketManager &Salmon::getSocketManager() const {
  return *(this->socketManager);
}

// See header file.
void Salmon::run() {
  // Creates a new ConfigData object and loads the configuration data.
  // :TODO: Consider changing it from a relative path to one specified by the 
  //        client. Possibly through a flag?
  this->configData = new ConfigData("files/configuration.config");

  // Creates a new ThreadPool.
  this->threadPool = new ThreadPool(strtos(this->getConfigData()["threadpool.poolsize"]));

  // Creates a new SocketManager and begins listening for incomming connections.
  this->socketManager = new SocketManager(this->getConfigData()["game.ipaddress"], 
                                          strtos(this->getConfigData()["game.port"]), 
                                          strtos(this->getConfigData()["game.backlog"]));

  // Begins reacting to incoming connections. Blocks the main thread here.
  this->getSocketManager().getConnectionReactor().handleConnections();
}